
from urllib.parse import urlencode, urlparse
from odoo.tests import HttpCase, tagged

@tagged('post_install', '-at_install')
class TestWebsiteSalarySimulation(HttpCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.department_rd = cls.env["hr.department"].create({'name': 'R&D'})

        cls.structure_type = cls.env['hr.payroll.structure.type'].create({'name': 'struct'})
        cls.contract_tmpl_rd_dev = cls.env["hr.contract"].create({
            'name': 'R&D Python/JS developer',
            'employee_id': False,  # this is a template
            'department_id': cls.department_rd.id,
            'structure_type_id': cls.structure_type.id,
            'wage': 6500,
            'job_id': False,
        })

        cls.job_rd_dev = cls.env["hr.job"].create({
            'name': 'R&D Python/JS developer',
        })

    def _simulation_link(self, contract_id, **kwargs):
        slink = "/salary_package/simulation/contract/%d" % contract_id
        if kwargs:
            slink = "%s?%s" % (slink, urlencode(kwargs))
        return slink

    def test_salary_simulation_redirect(self):
        """Ensure salary simulation redirect to a simulation offer."""
        last_offer_id = self.env["hr.contract.salary.offer"].search([], order='id DESC', limit=1).id or 0

        # check that job_id is mandatory
        resp = self.url_open(self._simulation_link(self.contract_tmpl_rd_dev.id))
        resp.raise_for_status()
        self.assertIn("This offer is outdated, please request an updated link", resp.text)

        # check that we create a new offer on-demand and redirect to it
        resp = self.url_open(
            self._simulation_link(self.contract_tmpl_rd_dev, job_id=self.job_rd_dev.id, final_yearly_costs=64000),
            allow_redirects=False,
        )
        resp.raise_for_status()
        self.assertEqual(resp.status_code, 303)

        new_simulation_salary_offer = self.env["hr.contract.salary.offer"].search([('id', '>', last_offer_id)])
        self.assertEqual(
            len(new_simulation_salary_offer), 1,
            "We should have create a new salary simulation",
        )
        self.assertEqual(new_simulation_salary_offer.contract_template_id, self.contract_tmpl_rd_dev)
        self.assertEqual(new_simulation_salary_offer.employee_job_id, self.job_rd_dev)
        self.assertFalse(new_simulation_salary_offer.employee_id)
        self.assertFalse(new_simulation_salary_offer.applicant_id)
        self.assertEqual(new_simulation_salary_offer.final_yearly_costs, 64000)
        self.assertEqual(new_simulation_salary_offer.state, 'open')

        redirect_urlp = urlparse(resp.headers['Location'])
        self.assertEqual(
            redirect_urlp.path,
            "/salary_package/simulation/offer/%s" % new_simulation_salary_offer.id,
            "We should redirect to simulation offer",
        )
        self.assertEqual(
            redirect_urlp.query,
            urlencode({'job_id': self.job_rd_dev.id}),
            "We have to keep job_id in query on redirect so that controller know it's for a new applicant",
        )

    def test_salary_simulation_reuse_existing_simulation_offer(self):
        """Ensure salary simulation reuse existing offer between calls."""
        last_offer_id = self.env["hr.contract.salary.offer"].search([], order='id DESC', limit=1).id or 0
        salary_simulation_link = self._simulation_link(
            self.contract_tmpl_rd_dev, job_id=self.job_rd_dev.id, final_yearly_costs=64000
        )

        # First simulation request
        resp = self.url_open(salary_simulation_link, allow_redirects=False)
        resp.raise_for_status()

        new_simulation_salary_offer = self.env["hr.contract.salary.offer"].search([('id', '>', last_offer_id)])
        self.assertEqual(len(new_simulation_salary_offer), 1, "We should have create one offer")

        # Second simulation request
        resp2 = self.url_open(salary_simulation_link, allow_redirects=False)
        resp2.raise_for_status()

        other_offers = self.env["hr.contract.salary.offer"].search([('id', '>', new_simulation_salary_offer.id)])
        self.assertFalse(other_offers, "We should not have created any new offer")
        self.assertEqual(
            resp.headers["Location"],
            resp2.headers["Location"],
            "We should have reused the preceding offer and should redirect to the same location",
        )

    def test_salary_simulation_no_crash_if_multiple_offer(self):
        """Ensure we don't crash if multiple simulation offer exists for the same contract."""
        last_offer_id = self.env["hr.contract.salary.offer"].search([], order='id DESC', limit=1).id or 0
        salary_simulation_link = self._simulation_link(
            self.contract_tmpl_rd_dev, job_id=self.job_rd_dev.id, final_yearly_costs=64000
        )

        resp = self.url_open(salary_simulation_link, allow_redirects=False)
        resp.raise_for_status()
        new_simulation_salary_offer = self.env["hr.contract.salary.offer"].search([('id', '>', last_offer_id)])

        # create a duplicate
        dup_simulation_salary_offer = new_simulation_salary_offer.copy()
        resp2 = self.url_open(salary_simulation_link, allow_redirects=False)
        resp2.raise_for_status()
        self.assertEqual(resp2.status_code, 303)
        self.assertEqual(
            urlparse(resp2.headers['Location']).path,
            "/salary_package/simulation/offer/%s" % dup_simulation_salary_offer.id,
            "In case of duplicate simulation offer we should prefer the most recent one",
        )
